/*
 *    interface.c
 *      Rozhrani s uzivatelem
 *
 */

#include<stdio.h>
#include"data.h"

extern TMove historie[MAXMOVHIS];	/* 1.tah = [1] */
extern int IQ, SIQ;

void PrintMoves(TMoveList mg, int p)
{
	int i;
	char t[5];
	printf("Moves possible (%d): ", p);
	for (i = 0; i < p; i++) {
		SquareToStr(mg[i].from, &t[0]);
		SquareToStr(mg[i].to, &t[2]);
		printf("%s ", t);
	}
	printf("\n");
}

void PrintBoard(TGame H)
{
	int x, y;
	printf(" \n  -----------------------------------------\n");
	for (y = 7; y >= 0; y--) {
		printf("%d |", y + 1);
		for (x = 0; x <= 7; x++)
			switch (H.board[x][y]) {
			case EMPT:
				printf("    |");
				break;
			case WPWN:
				printf("  P |");
				break;
			case WBSP:
				printf("  B |");
				break;
			case WKNT:
				printf("  N |");
				break;
			case WROK:
				printf("  R |");
				break;
			case WQEN:
				printf("  Q |");
				break;
			case WKNG:
				printf("  K |");
				break;
			case BPWN:
				printf(" *P |");
				break;
			case BBSP:
				printf(" *B |");
				break;
			case BKNT:
				printf(" *N |");
				break;
			case BROK:
				printf(" *R |");
				break;
			case BQEN:
				printf(" *Q |");
				break;
			case BKNG:
				printf(" *K |");
				break;
			default:
				printf("  F");
			}
		printf(" \n  -----------------------------------------\n");
	}
	printf("     a    b    c    d    e    f    g    h \n");
}

void PrintHist(TGame H)
{
	int i;
	char odc[3], doc[3];
	for (i = 1; i <= H.PHT; i++) {
		SquareToStr(historie[i].from, odc);
		SquareToStr(historie[i].to, doc);
		if (i % 2 == 1)
			printf("%d. %2.2s-%2.2s", (i + 1) / 2, odc, doc);
		else
			printf("  %2.2s-%2.2s\n", odc, doc);
	}
	if (H.PHT % 2 == 1)
		printf("\n");
}

void PrintPieces(TGame H)
/* vytiskne pozice vsech figur */
{
	int i, x;
	char notpoz[3];
	TPlayer *hrac;
	printf("DEBUG: Pozice vsech figur:\n");
	hrac = &(H.whiteplayer);
	printf(" whiteplayer: ");
	for (x = 0; x <= 1; x++) {
		for (i = 0; i <= 15; i++) {
			switch (hrac->pieces[i].fig) {
			case WPWN:
				printf("  P");
				break;
			case WBSP:
				printf("  S");
				break;
			case WKNT:
				printf("  J");
				break;
			case WROK:
				printf("  V");
				break;
			case WQEN:
				printf("  Q");
				break;
			case WKNG:
				printf("  K");
				break;
			case BPWN:
				printf(" *P");
				break;
			case BBSP:
				printf(" *S");
				break;
			case BKNT:
				printf(" *J");
				break;
			case BROK:
				printf(" *V");
				break;
			case BQEN:
				printf(" *Q");
				break;
			case BKNG:
				printf(" *K");
				break;
			}
			if ((hrac->pieces[i].fig) != EMPT) {
				SquareToStr(hrac->pieces[i].pos, notpoz);
				printf(" :%s ; ", notpoz);
			}
		}		/* for i */
		if (x == 0) {
			hrac = &(H.blackplayer);
			printf(" \nCERNY: ");
		}
		/* prohodit hrace + newline */
		else
			printf(" \n");
	}			/* for x */
}

void SquareToStr(TSquare p, char *ch)
{
	ch[0] = (char) (p.x + (int) ('a'));
	ch[1] = (char) (p.y + (int) ('1'));
	ch[2] = '\0';
}

void PromotionToStr(char p, char *ch) // Roland
{
  if ((p == WQEN) || (p == BQEN)) {
    ch[0] = 'q';
    ch[1] = '\0';
  } else
  {
    ch[0] = '\0';
    ch[1] = '\0';
  }
}

void Not2Num(char *ch, TSquare * p)
{
	p->x = (int) (ch[0] - 'a');
	p->y = (int) (ch[1] - '1');
}

void GameOver(void)
{
	printf(" quiting ...\n");
}

/* soubory */

int ReadCFG(void)
{
	int k, l;
	FILE *f;

	f = fopen(CFGFILE, "rt");
	if (f == NULL) {
		printf("Warning: error reading configuration file %s: ", CFGFILE);
		perror(NULL);
		printf("Using default values\n");
		return 0;
	} else {
		fscanf(f, "%d %d", &k, &l);
		if (k > 0 && l <= 0) {
			IQ = k;
			SIQ = l;
		} else {
			printf("Incorrect values in %s", CFGFILE);
			return 0;
		}
	}
	return 1;
}

void SaveFile(TGame * H, char *fname)
{
	FILE *f;
	int i;
	char t[5];
	f = fopen(fname, "wt");
	if (f == NULL)
		perror("Error saving file");
	else {
		fprintf(f, "%d\n", H->PHT);
		for (i = 1; i <= H->PHT; i++) {
			SquareToStr(historie[i].from, &t[0]);
			SquareToStr(historie[i].to, &t[2]);
			fprintf(f, "%d %s\n", i, t);
		}
		fclose(f);
		printf("%s written.\n", fname);
	}
}

void OpenFile(TGame * H, char *fname)
{
	FILE *f;
	int i, j, pht;
	char t[5];
	InitGame(H);
	f = fopen(fname, "rt");
	if (f == NULL)
		perror("Error opening file");
	else {
		fscanf(f, "%d\n", &pht);	/* j -> zahodit ... */
		for (i = 1; i <= pht; i++) {
			if (!feof(f)) {
				fscanf(f, "%d %s\n", &j, t);
				if (j != i)
					printf("%d: Nonconsistent data\n", i);
				Not2Num(&(t[0]), &(historie[i].from));
				Not2Num(&(t[2]), &(historie[i].to));
				MakeMove(t, H);
			} else
				printf("%d: Nonconsistent data\n", i);
		}
		fclose(f);
		printf("%s opened.\n", fname);
	}
}
