/*
 *    MAIN.C
 *
 *      sachovy program pro unix - ver. 0.1b
 *      ========================
 *
 *      Martin Macok
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "data.h"

#ifdef UNIX
#include <signal.h>
#endif				/*  */

void undo(TGame *);		/* vezme zpet posledni tah */

int IQ = DEPTH, SIQ = SDEPTH;
TMove historie[MAXMOVHIS];	/* 1.tah = [1] */

int main(int argc, char **argv)
{
	TGame game;
	char lCmd[10], fn[30], movestr[/*5*/6], nextchar;
	TMoveList mg;
	int i, XB = 1, comp = BLACK, doba, IB, pt, vt;
  int tseconds2;
	TBook book;
	time_t seeed;

	setbuf(stdin, NULL);	/* bacha na to! muze to zpusobit problemy */
	/* udajne tam stejne bude jedno-char-ovej buffer, protoze
	   scanf nekdy potrebuje vracet jeden znak zpet na stdin */
	/* toz radsi budem jeste flushovat */
	setbuf(stdout, NULL);	/* kvuli XBOARD -> unbuffered stdout */
	printf("\n *** Sachy (simple chess program) *** (v%s)\n\n", VERSION);
#ifdef UNIX
	signal(SIGINT, SIG_IGN);
#endif
	time(&seeed);
	srand((unsigned int) seeed);
	InitGame(&game);
	//ReadCFG();
	if (openbook(&book))
		IB = 1;
	/* printbook(&book); */
	else
		IB = 0;
	do {
		/*      fflush(stdin); fflush(stdout); */
		if (IsStaleMate(&game)) {
			if (XB) printf("\007 Draw !!!\n");
			/*              GameOver();
			   return 0; */
		} else if (IsMate(&game)) {
			if (XB) printf("\007 Checkmated !!!\n");
			/*              GameOver();
			   return 0; */
		} else if (InCheck(game.sidetomove, &game))
			if (XB) printf(" check!\n");
		if (XB) printf("(%d)", (game.PHT) / 2 + 1);
		if (game.sidetomove == WHITE) {
            if (XB) printf(" white> ");
        } else
			if (XB) printf(" black> ");
		if (game.sidetomove == comp) {
			if (XB) printf(" ... thinking ...");
			if (IB) {
				pt = book.p[book.aktual].poctah;
				if (pt > 0) {
					vt = rand() % pt;
					strcpy(movestr, book.p[book.aktual].tahy[vt]);
					book.aktual = book.p[book.aktual].to[vt];
					doba = 0;
				} else
					IB = 0;
			}
			if (IB == 0)
				doba = BestMove(&game, movestr, XB);
			if (doba >= 0) {
        game.tleft -= doba;
        if (((game.PHT+3)/2) == game.tmoves)
          game.tleft += game.tseconds;
				MakeMove(movestr, &game);
				if (XB) {
					PrintBoard(game);
          printf("time left: %d seconds\n", game.tleft);
        }
				printf("move %s\n", movestr);
			} else
				comp = -1;
			continue;
		}
		scanf("%9s", lCmd);
		if (strcmp(lCmd, "new") == 0) {
			InitGame(&game);
			comp = BLACK;
			IB = 1;
			book.aktual = 0;
		} else if (strcmp(lCmd, "random") == 0 ||
			   strcmp(lCmd, "?") == 0)
			continue;
		else if (strcmp(lCmd, "book") == 0)
			printbook(&book);
		else if (strcmp(lCmd, "xboard") == 0) {
			XB = 0;
			printf("\n");
		} else if (strcmp(lCmd, "draw") == 0)
            { if (XB) printf("Go on!\n"); }
		else if (strcmp(lCmd, "white") == 0) {
			comp = BLACK;
			game.sidetomove = WHITE;
		} else if (strcmp(lCmd, "black") == 0) {
			comp = WHITE;
			game.sidetomove = BLACK;
		} else if (strcmp(lCmd, "force") == 0)
			comp = -1;
		else if (strcmp(lCmd, "undo") == 0) {
			undo(&game);
			IB = 0;
		} else if (strcmp(lCmd, "remove") == 0) {
			undo(&game);
			undo(&game);
			IB = 0;
		} else if (strcmp(lCmd, "tisk") == 0)
			PrintBoard(game);
		else if (strcmp(lCmd, "test") == 0)
			printf("Benchmark: %d positions per second\n", benchmark(&game));
		else if (strcmp(lCmd, "poz") == 0)
			PrintPieces(game);
		else if (strcmp(lCmd, "hod") == 0)
			printf("%d\n", Evaluate(&game));
		else if (strcmp(lCmd, "hist") == 0)
			PrintHist(game);
		else if (strcmp(lCmd, "slist") == 0) {
			i = SMoveGen(&game, mg);
			PrintMoves(mg, i);
		} else if (strcmp(lCmd, "list") == 0) {
			i = MoveGen(&game, mg, 0);
			PrintMoves(mg, i);
		} else if (strcmp(lCmd, "hint") == 0) {
			doba = BestMove(&game, movestr, XB);
			if (doba > 0)
				printf("Hint: %s\n", movestr);
		} else if (strcmp(lCmd, "go") == 0)
			comp = game.sidetomove;
		else if (strcmp(lCmd, "ply") == 0) {
			scanf("%d", &i);
			if (i > 0)
				IQ = i;
		} else if (strcmp(lCmd, "sply") == 0) {
			scanf("%d", &i);
			if (i <= 0)
				SIQ = i;
		} else if (strcmp(lCmd, "open") == 0) {
			scanf("%29s", fn);
			OpenFile(&game, fn);
			IB = 0;
		} else if (strcmp(lCmd, "save") == 0) {
			scanf("%29s", fn);
			SaveFile(&game, fn);
		} else if (strcmp(lCmd, "quit") == 0) {
			GameOver();
			break;
        } else if (strcmp(lCmd, "level") == 0) {
            scanf("%d %d", &game.tmoves, &game.tseconds);
            game.tseconds *= 60;
            scanf("%c", &nextchar);
            if (nextchar == ':') {
                scanf("%d %d", &tseconds2, &game.tclockmode);
                game.tseconds += tseconds2;
            } else if (nextchar == ' ') {
                scanf("%d", &game.tclockmode);
            } else
                printf("Error parsing level command.");
            if (game.tclockmode != 0) {
                printf("Error: only conventional clock mode is supported\n");
            }
            game.tleft = game.tseconds;
		} else if (IsMoveStr(lCmd)) {
			if (IsPossible(lCmd, &game) && IsLegalMove(lCmd, &game, 1)) {
				MakeMove(lCmd, &game);
				if (IB) {
					pt = book.p[book.aktual].poctah;
					i = 0;
					while (i < pt &&
					       strcmp(lCmd, book.p[book.aktual].tahy[i]) != 0)
						i++;
					if (i < pt)
						book.aktual = book.p[book.aktual].to[i];
					else
						IB = 0;
				}
				if (XB)
					PrintBoard(game);
			} else
				printf("%s\n", lCmd);
		} else
			printf("Error (unknown command): %s \n", lCmd);
	}
	while (1);
	return 0;
}

void undo(TGame * H)
{
	int i, pht;
	char t[5];

	pht = H->PHT;
	InitGame(H);
	for (i = 1; i < pht; i++) {
		SquareToStr(historie[i].from, &(t[0]));
		SquareToStr(historie[i].to, &(t[2]));
		MakeMove(t, H);
	}
}
